/***************************************************************************
	This source code is part of the Cognachrome 2000 vision system.
	Copyright 1994, Anne Wright, anarch@mit.edu,
	                Randy Sargent, rsargent@mit.edu
	This copy is not to be distributed in any form
	and is to be used only with express permission of the author.
****************************************************************************/

#ifndef __vision__
#define __vision__

#include "visiontypes.h"

#define UNKNOWN 0xffff
#define NUM_CHANNELS 8

/**************************************************************************/
/* Defines for framing */
#define TPU_WIDTH 		267
#define TPU_HEIGHT 		256

#define FRAME_WIDTH  		64
#define FRAME_HEIGHT 		48

#define LUT_WIDTH 		256
#define LUT_HEIGHT 		256

#define FRAMEGRAB_LEFT_BORDER 	8
#define FRAMEGRAB_TOP_BORDER  	12
#define FRAMEGRAB_LINE_SPACING 	5

#define TPU_TOP_EDGE		(FRAMEGRAB_TOP_BORDER)
#define TPU_BOTTOM_EDGE		(FRAMEGRAB_TOP_BORDER + \
				 FRAMEGRAB_LINE_SPACING * (FRAME_HEIGHT))
#define TPU_FRAME_HEIGHT	(TPU_BOTTOM_EDGE-TPU_TOP_EDGE)

#define TPU_LEFT_EDGE		(*tpu_left_edge)  /* temporary hack */
#define TPU_RIGHT_EDGE		(*tpu_right_edge) /* temporary hack */
#define TPU_FRAME_WIDTH		(TPU_RIGHT_EDGE-TPU_LEFT_EDGE)

#define TPU_TO_FRAME_ROW(tr) 	(((((tr)-TPU_TOP_EDGE)*(FRAME_HEIGHT)) + \
				  (FRAME_HEIGHT/2)) / \
				 TPU_FRAME_HEIGHT)
#define TPU_TO_FRAME_COL(tc) 	(((((tc)-TPU_LEFT_EDGE)*(FRAME_WIDTH)) + \
				  (FRAME_WIDTH/2)) / \
				 TPU_FRAME_WIDTH)
#define FRAME_TO_TPU_ROW(fr)	(((fr)*FRAMEGRAB_LINE_SPACING)+ \
				 FRAMEGRAB_TOP_BORDER+ \
				 (FRAMEGRAB_LINE_SPACING/2))
#define FRAME_TO_TPU_COL(fc)	((((fc)*(TPU_FRAME_WIDTH))/(FRAME_WIDTH)) + \
				 TPU_LEFT_EDGE)

#define CROSS_LEFT 46
#define CROSS_RIGHT 267
#define CROSS_WIDTH 221

#define HSYNC_THRESH 240

#define lut_base 	0x400000
#define latch_base	0x500000
#define color_base  	0x500000
#define outbuff_base  	0x600000

#define OUTBUFF_CS	3
#define LUT_CS		4
#define COLOR_CS	10	/* was 5d */

/* Vesta TPU serial port assignments:
   (board version 1.1)
   tpu1:  port b rxd
   tpu4:  port b txd
   tpu5:  serial chip enable  (if you disconnect J10 on rev 1.2 and later,
                               the serial chip is always enabled and tpu5
			       is free) 
   */
   
#define HSYNC_CH	0
#define SERIALB_RXD	1

#define IDROP_C_CH 	2

#define OVERLAY_CH	3
#define SERIALB_TXD     4

#define IRIS_C_CH	5

#define IDROP_B_CH 	6

#define UNUSED_B_CH	7
#define CAR_SERVO_CH    7    /* for car */

#define IRIS_B_CH 	8

#define BEEPER_CH 	9

#define IDROP_A_CH 	10

#define UNUSED_A_CH	11
#define CAR_LEFT_MOTOR_CH 11  /* for car */

#define IRIS_A_CH 	12

#define IR_CHANNEL	13

#define VSYNC_CH	14

#define UNUSED_15_CH	15
#define CAR_RIGHT_MOTOR_CH 15  /* for car */

#define LUT_DSACK	2


/* HSYNC_BIT set to input (used) */
/* DSACK0 used for bus control */
/* Bit 2 (AVEC) is frobbed in interrupt code (should take this out eventually) */
/* i2c data bit is used */
/*
   potentially free:
   dsack1   5v
   ds       0v
   as       0v
   siz0     nope
   */
   

#define DSACK0		0x1	/* dsack0 */

#define DSACK1		0x2	/* dsack1 */
#define PORTE_CAR_LEFT_MOTOR_DIR  0x02

#define AVEC		0x4	/* avec */
#define PORTE_CAR_RIGHT_MOTOR_DIR  0x04

#define HSYNC_BIT       0x8 	/* rmc */

#define DS		0x10	/* ds */
#define PORTE_CAR_ENABLE  0x10
/* was used for debugging, commented out but untested */

#define AS	 	0x20	/* as */
/* was used for debugging, commented out but untested */

#define SIZ0	 	0x40	/* siz0 */

#define I2C_DAT_BIT 	0x80    /* siz1 */

#define HSYNC() 	(sim->porte & HSYNC_BIT)
#define hcount_ptr      ((byte *)(0xffffec))

#define OUTLATCH ((Output_latch *)latch_base)

extern Output_latch * const outlatch;

/********************* i2c macros *************************************/
#define I2C_DAT_PORT		(sim->porte)
#define SET_I2C_DAT(val) 	((val)?(I2C_DAT_PORT |= I2C_DAT_BIT):\
				 (I2C_DAT_PORT &= (I2C_DAT_BIT ^ 0xff)))
#define GET_I2C_DAT()		((I2C_DAT_PORT & I2C_DAT_BIT) ? 1 : 0)
#define SET_I2C_CLK(val) 	(OUTLATCH->i2c_clk=val)
#define GET_I2C_CLK()		(0)
#define SET_I2C_INPUT()		(set_porte_input(I2C_DAT_BIT))
#define SET_I2C_OUTPUT()	(set_porte_output(I2C_DAT_BIT))


static inline void set_filter(int val)
{
   OUTLATCH->m= ((val >> 1) & 1);
   OUTLATCH->n= ((val >> 0) & 1);
}
static inline void filter_rise(int n)
{
  OUTLATCH->n=n;
}
static inline void set_video_mux(int outmux)
{
  OUTLATCH->mux0=outmux;
  OUTLATCH->mux1=outmux>>1;
}
static inline void LUT_MODE()
{
  OUTLATCH->digitize=0;
  OUTLATCH->recog=0;
}
static inline void DIGITIZE_MODE()
{
  OUTLATCH->digitize=1;
  OUTLATCH->recog=0;
}
static inline void RECOG_MODE()
{
  OUTLATCH->digitize=1;
  OUTLATCH->recog=1;
}

#include "tpuvis.h"
#include "vis_hw.h"
#include "frame.h"
#include "i2c.h"
#include "eep.h"
#include "lut.h"
#include "sony.h"
#include "blob.h"
#include "intvision.h"
#include "protocol.h"
#include "beep.h"
#include "colorepts.h"
#include "colorplot.h"
#include "visdraw.h"
#include "ir.h"
#include "protocol.h"
#include "beep.h"
#include "vstream.h"
#include "visui.h"
#include "vision.pro"
/* The version is defined in the Makefile now. */
/* #include "version.h" */ 
#include "frameproc.h"
#include "intmath.pro"
#include "manual_lut.h"

extern FILE *vision_stream;
extern persistent filt_level;

extern Persistent tpu_left_edge,tpu_right_edge; /* temporary hack */

#endif


