/***************************************************************************
	This source code is part of the Cognachrome 2000 vision system.
	Copyright 1994, Anne Wright, anarch@mit.edu,
	                Randy Sargent, rsargent@mit.edu
	This copy is not to be distributed in any form
	and is to be used only with express permission of the author.
****************************************************************************/


typedef volatile struct vsync_struct {
   channel_control ctl;			/* prm0 */
   word blank0;				/* prm1 */
   word blank1;				/* prm2 */
   word lastdir;			/* prm3 */
   word uptime;				/* prm4 */
   word downtime;			/* prm5 */
} *vsync;

typedef volatile struct hsync_struct {
   channel_control ctl;
   word vert_thresh;
   word last_hsync;
   word row;
} *hsync;

typedef volatile struct vcross_struct {
   channel_control ctl;			/* prm0 */
   word blank0;				/* prm1 */
   word blank1;				/* prm2 */
   word hwidth;				/* prm3 */
   word row;				/* prm4 */
   word col;				/* prm5 */
} *vcross;


typedef struct edge_dat_struct {
   word row;
   byte e;  byte b;
} edge_dat;

typedef struct edge_primary_struct {
   channel_control ctl;
   word edge_rise_ch;
   edge_dat ed2;
   edge_dat ed3;
} *edge_primary;

typedef struct edge_secondary_struct {
   /* first word doubles as ctl on init */
   edge_dat ed0;
   edge_dat ed1;
   edge_dat ed4;
} *edge_secondary;

typedef unsigned char uchar;
typedef void Intfunc();

typedef volatile struct output_latch_str{
  word mux1;
  word mux0;
  word m;
  word n;
  word i2c_clk;
  word recog;
  word overlay;
  word digitize;} Output_latch;

typedef struct color24_structure {
  byte g;
  byte b;
  byte g2;
  byte r;} Color24;

typedef struct color16_structure {
  byte g;
  byte rb;} Color16;

typedef byte Color8;

typedef enum color_enum {blue,red,green} Color;

typedef struct frame24_str {
  int width;
  int height;
  Color24 buff[1];} Frame24;

typedef struct frame16_str{
  int width;
  int height;
  Color16 buff[1];} Frame16;

typedef struct frame8_str{
  int width;
  int height;
  Color8 buff[1];} Frame8;

typedef struct frame1_str{
  int width;
  int height;
  int lwidth;
  uint buff[1];} Frame1;

typedef struct epts_str {
   short row;
   byte e;  byte b;
   short next;
   short blob;
} Epts;

typedef struct colorepts_str {
   int top;
   int bottom;
   Epts earr[256];
} Colorepts;

typedef struct blob_str {
   struct edge_buffer_str *eb;
   struct blob_str *merged_with;
   short first;
   short last;
   long  area;
   short  xcg;
   short  ycg;
   long xcg_scaled;
   long ycg_scaled;
   unsigned char left, right, top, bottom;
   short  angle;
   short  aspect;
   short  n_edges;
} Blob;


#define ED_LEN 1000
#define MAXEPTS 800
#define STARTBLOBS 10

enum eb_status {
   eb_grabbing,
   eb_finished_grabbing,
   eb_done
  };

#define MAX_ACTIVE_BLOBS 10
#define MAX_DONE_BLOBS 50
#define MAX_BLOBS (60*3)

typedef struct edge_buffer_str {
   Epts volatile *ptr;   /* MUST BE ELT 0: accessed in assembly code */
   Epts *limit; /* MUST BE ELT 1: accessed in assembly code */
                    /* actual limit minus 6 items */
   Epts *start;
   volatile enum eb_status status;
   int frame_count;
   int frame_mtime;
   int frames_missed;

   struct epts_str *next_processed_edge;
   short nactive;
   struct blob_str *active[MAX_ACTIVE_BLOBS];
   short ndone;
   struct blob_str *done[MAX_DONE_BLOBS];
} Edge_buffer;

typedef struct vstate_str {
   volatile int acquire_trigger;
   volatile int acquire_terminate;
   volatile int acquiring_frames;
   volatile int latch;	       /* which buffer is being grabbed into */

   int iris_ch;
   int idrop_ch;
   
   edge_secondary iris;
   edge_primary idrop;
   
   Intfunc *f;

   int frame;			/* which frame is being processed */
   struct edge_buffer_str raw[2];      /* double buffering state */
   struct edge_buffer_str **geb;
   
   int min_size;		/* threshold for blob size */
   char *protocol_format;
   char *protocol_args;
   int  proclev;
   short max_objs;
   short min_objs;
} Vstate;

typedef int Frame_function(Vstate *state,Edge_buffer *ed);
